
# q-expansion of newform 1369.2.b.d, downloaded from the LMFDB on 10 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1369
weight = 2
poly_data = [1, 0, -1, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 0, 1], 1], [[-1, 0, 2, 0], 1], [[0, 2, 0, -1], 1]]

hecke_ring_character_values = [[2, [-1, 0, 0, 0]]]
aps_data = [[0, 1, 0, 0], [-1, 0, 0, 1], [0, 2, 1, 0], [0, 0, 0, -2], [-3, 0, 0, 1], [0, 0, -2, 0], [0, -2, 1, 0], [0, 3, 1, 0], [0, -1, -1, 0], [0, -2, -1, 0], [0, 3, -3, 0], [0, 0, 0, 0], [3, 0, 0, 0], [0, 0, 2, 0], [3, 0, 0, 3], [-6, 0, 0, -2], [0, 10, -2, 0], [0, 6, 1, 0], [-9, 0, 0, -1], [6, 0, 0, 0], [0, 0, 0, 0], [0, 9, 3, 0], [-3, 0, 0, 3], [0, -4, 3, 0], [0, 6, 1, 0], [-9, 0, 0, 0], [0, 6, -4, 0], [-6, 0, 0, 4], [0, 12, 3, 0], [0, 8, 2, 0], [-7, 0, 0, 3], [0, -5, -9, 0], [-3, 0, 0, 2], [-3, 0, 0, -9], [9, 0, 0, 2], [1, 0, 0, -9], [3, 0, 0, 10], [0, -12, 6, 0], [0, 4, -4, 0], [3, 0, 0, -4], [0, 2, 2, 0], [3, 0, 0, -4], [0, 1, -3, 0], [0, 6, -5, 0], [21, 0, 0, -4], [0, 18, 4, 0], [-10, 0, 0, 0], [-11, 0, 0, -9], [0, 11, -7, 0], [7, 0, 0, 0], [3, 0, 0, 8], [0, -10, 4, 0], [0, -12, 2, 0], [0, 2, 16, 0], [0, -4, 9, 0], [-18, 0, 0, -6], [18, 0, 0, 2], [-3, 0, 0, 5], [0, -12, -1, 0], [0, -28, 3, 0], [0, 0, -6, 0], [15, 0, 0, -8], [-4, 0, 0, 6], [0, 20, -6, 0], [0, 0, -9, 0], [-9, 0, 0, 4], [0, -12, 12, 0], [19, 0, 0, -6], [0, -11, 1, 0], [15, 0, 0, 4], [0, -14, -1, 0], [6, 0, 0, -6], [-10, 0, 0, 6], [21, 0, 0, 2], [12, 0, 0, 2], [0, -2, 8, 0], [0, 26, -3, 0], [3, 0, 0, 14], [0, 16, 6, 0], [0, 0, 5, 0], [0, 0, 0, -10], [0, -18, -1, 0], [0, 11, 13, 0], [27, 0, 0, 0], [0, 18, 2, 0], [30, 0, 0, -2], [0, -8, 14, 0], [0, -24, 7, 0], [0, 20, 2, 0], [0, -12, 2, 0], [0, -14, -10, 0], [0, 22, -10, 0], [0, -3, 17, 0], [33, 0, 0, 1], [0, -3, -9, 0], [0, -17, -15, 0], [15, 0, 0, 12], [0, 0, 0, -4], [0, -3, -11, 0], [0, 6, 1, 0], [0, -9, -3, 0], [0, 16, -1, 0], [0, -19, 9, 0], [0, 14, -13, 0], [-9, 0, 0, 13], [0, -6, 4, 0], [0, -25, -5, 0], [-3, 0, 0, -12], [27, 0, 0, 11], [21, 0, 0, -6], [0, 3, 5, 0], [-7, 0, 0, -6], [6, 0, 0, 18], [-14, 0, 0, -12], [0, -21, -1, 0], [3, 0, 0, 12], [0, 3, 5, 0], [0, 40, -2, 0], [0, -8, -3, 0], [6, 0, 0, -2], [0, -18, -3, 0], [-12, 0, 0, -16], [-27, 0, 0, 10], [0, -38, -4, 0], [-3, 0, 0, -11], [0, -8, -8, 0], [0, 30, 0, 0], [-3, 0, 0, -17], [0, 12, 8, 0], [8, 0, 0, -12], [-29, 0, 0, -3], [-45, 0, 0, -1], [-30, 0, 0, -12], [0, -6, 21, 0], [-21, 0, 0, -10], [0, -24, -2, 0], [-3, 0, 0, 26], [-10, 0, 0, -6], [0, 8, 0, 0], [0, 20, -8, 0], [-6, 0, 0, -14], [18, 0, 0, -18], [-29, 0, 0, -15], [0, -16, 4, 0], [0, 48, 2, 0], [-21, 0, 0, 5], [0, -30, 7, 0], [0, -22, 1, 0], [0, 12, 2, 0], [42, 0, 0, 2], [-15, 0, 0, -12], [15, 0, 0, -10], [0, -27, 5, 0], [3, 0, 0, -15], [0, 3, 1, 0], [0, 31, -13, 0], [0, 15, 11, 0], [-27, 0, 0, 2], [-23, 0, 0, -6], [27, 0, 0, 18], [0, 38, 6, 0], [-18, 0, 0, -8], [0, 12, -12, 0], [-3, 0, 0, 3], [0, -8, -14, 0], [45, 0, 0, -7], [0, 36, -6, 0], [0, -24, -2, 0], [-5, 0, 0, 6], [0, 10, 7, 0], [0, -4, -14, 0], [0, 18, -5, 0], [0, 31, 11, 0], [-5, 0, 0, 18], [-18, 0, 0, 12], [0, -26, -20, 0], [0, -24, -8, 0], [45, 0, 0, 8], [-4, 0, 0, 12], [-25, 0, 0, -12], [0, 15, 19, 0], [0, -19, -3, 0], [0, 30, 1, 0], [0, 20, 5, 0], [-9, 0, 0, -1], [-3, 0, 0, -22], [7, 0, 0, 0], [0, -18, 14, 0], [0, -6, 9, 0], [-33, 0, 0, -5], [0, -6, -5, 0], [24, 0, 0, -4], [0, -3, 9, 0], [-3, 0, 0, 8], [12, 0, 0, 10], [-21, 0, 0, -18], [0, 6, 35, 0], [0, -54, 0, 0], [-18, 0, 0, -10], [0, 35, 15, 0], [0, 10, 12, 0], [16, 0, 0, 12], [1, 0, 0, 0], [3, 0, 0, -2], [18, 0, 0, -6], [0, 16, -27, 0], [-3, 0, 0, 11], [-48, 0, 0, -8], [0, -20, 1, 0], [39, 0, 0, 1], [0, -36, 0, 0], [0, 22, 3, 0], [0, 6, 32, 0], [-6, 0, 0, 20], [0, -61, 3, 0], [-54, 0, 0, -2], [0, -27, 5, 0], [0, -22, 25, 0], [0, 20, 0, 0], [6, 0, 0, 22], [18, 0, 0, -26], [-37, 0, 0, -9], [39, 0, 0, -14], [0, 24, -18, 0], [12, 0, 0, 10], [0, 24, -15, 0], [30, 0, 0, -10], [27, 0, 0, -13], [-15, 0, 0, -21], [0, 16, -18, 0], [-51, 0, 0, 14], [28, 0, 0, -24], [7, 0, 0, 15], [3, 0, 0, 32], [14, 0, 0, 0], [24, 0, 0, 26], [-32, 0, 0, 0], [0, 20, 9, 0], [0, -2, 26, 0], [0, 5, -13, 0], [0, -14, -2, 0], [0, -15, 21, 0], [21, 0, 0, -7], [0, -6, 11, 0], [-39, 0, 0, 10], [0, -20, 0, 0], [0, -12, -16, 0], [0, 17, 17, 0], [3, 0, 0, 33], [0, -5, -23, 0], [0, 54, -9, 0], [-9, 0, 0, 26], [9, 0, 0, -3], [0, 42, -12, 0], [0, -34, -4, 0], [15, 0, 0, -25], [-24, 0, 0, 0], [18, 0, 0, -16], [-9, 0, 0, -4], [0, 0, -11, 0], [0, -54, 0, 0], [0, -35, -5, 0], [14, 0, 0, -18], [-38, 0, 0, 18], [0, 20, -8, 0], [12, 0, 0, -30], [15, 0, 0, 34], [0, -38, 21, 0], [30, 0, 0, -6], [0, 8, -28, 0], [0, 12, -18, 0], [0, -15, -11, 0], [0, -36, 15, 0], [0, 3, 25, 0], [17, 0, 0, 36], [-4, 0, 0, 18], [-27, 0, 0, -27], [0, 6, -32, 0], [-25, 0, 0, -12], [0, -37, -11, 0], [-24, 0, 0, -10], [0, 18, 34, 0], [51, 0, 0, -3], [-20, 0, 0, 12], [0, 12, 42, 0], [12, 0, 0, 10], [0, -6, -13, 0], [30, 0, 0, -10], [0, 33, 9, 0], [0, -20, 31, 0], [0, 20, -6, 0], [0, 11, -15, 0], [15, 0, 0, -14], [-27, 0, 0, -15], [-21, 0, 0, -18], [-39, 0, 0, -16], [-36, 0, 0, 24], [33, 0, 0, 10], [0, 31, -31, 0], [6, 0, 0, -6], [0, 36, 23, 0], [42, 0, 0, 12], [-15, 0, 0, 11]]
